import mimetypes
import os
import pathlib
from copy import deepcopy

from django.contrib.auth.models import AbstractUser
from django.db import models
from django.template.defaultfilters import slugify
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.core.mail import EmailMultiAlternatives
from django.conf import settings
from django.utils.timezone import datetime, make_aware, make_naive

from phonenumber_field.modelfields import PhoneNumberField


class State(models.Model):
    name = models.CharField(_('name'), max_length=100, blank=False)
    abbrev = models.CharField(_('abbrev'), max_length=2, blank=True)
    hash = models.CharField(_('hash'), max_length=40, blank=True, null=True)
    display = models.BooleanField(default=True)
    atsReference = models.IntegerField(_('atsReference'), blank=True, null=True, default=None,
                                       help_text='ATS Reference', )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "States"
        app_label = 'project_center'


class City(models.Model):
    name = models.CharField(_('name'), max_length=100, blank=False)
    state = models.ForeignKey(State, null=True, blank=False, on_delete=models.SET_NULL)
    hash = models.CharField(_('hash'), max_length=40, blank=False, null=False)
    display = models.BooleanField(default=True)

    def __str__(self):
        try:
            return '{city}, {state}'.format(
                city=self.name,
                state=self.state.abbrev)
        except AttributeError as e:
            return str(self.name)

    class Meta:
        verbose_name_plural = "Cities"
        app_label = 'project_center'

    def list_zips(self):
        return [x.zipcode for x in Zip.objects.filter(city=self).order_by('-population_count')]


class Zip(models.Model):
    zipcode = models.CharField(primary_key=True, unique=True, max_length=5)
    city = models.ForeignKey(City, null=True, default=None, related_name='zip', on_delete=models.SET_NULL)
    approximate_latitude = models.FloatField(blank=True, null=True, default=None)
    approximate_longitude = models.FloatField(blank=True, null=True, default=None)
    population_count = models.IntegerField(blank=True, null=True)
    display = models.BooleanField(default=True)

    def __str__(self):
        try:
            return '{zipcode} ({city}, {state})'.format(
                zipcode=self.zipcode,
                city=self.city,
                state=self.city.state.name)
        except AttributeError as e:
            return str(self.zipcode)

    def get_geocode(self):
        return (self.approximate_latitude, self.approximate_longitude)

    def get_geolocation(self):
        return self.get_geocode()

    def get_es_geocode(self):
        return [self.approximate_longitude, self.approximate_latitude]


class Company(models.Model):
    name = models.CharField(_('Company Name'), max_length=100, blank=False)

    class Meta:
        verbose_name = 'Project Company'
        verbose_name_plural = "Project Companies"
        app_label = 'project_center'

    def __str__(self):
        return self.name


class User(AbstractUser):
    """A subclass of the user model for storing persistant candidate data as well as Admin Data."""

    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    title = models.CharField(_('Title'), max_length=64, blank=True, null=True, default='')
    address_1 = models.CharField(_('Address 1'), max_length=64, blank=True, null=True, default='')
    city = models.CharField(_('City'), max_length=64, blank=True, null=True, default='')
    state = models.ForeignKey(State, blank=True, null=True, default=None,
                              on_delete=models.SET_NULL)  # TODO: Add a validator
    postal_code = models.CharField(_('postal_code'), max_length=10, blank=True, null=True, default='',
                                   validators=[])  # TODO: Add a validator
    zip = models.CharField(_('postal_code'), max_length=10, blank=True, null=True, default='',
                           validators=[])  # TODO: Add a validator
    primary_phone = PhoneNumberField(_('primary_phone'), max_length=50, blank=True, null=True, default=None)
    company = models.ForeignKey(Company, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    email_notify = models.BooleanField(default=False)

    def __str__(self):
        return self.get_full_name()

    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project User')
        verbose_name_plural = _('Project Users')

    def _get_FIELD_display(self, field):
        field = super()._get_FIELD_display(field)
        return field

    def send_project_alert_email(self, subject=None, text=None, html=None, sender=None):

        subject, from_email, to = \
            subject if subject else 'Project Alert', \
            sender if sender else settings.EMAIL_DEFAULT_ADMIN_EMAIL_ADDRESS, \
                self.email
        text_content = text if text else 'A Project Activity Occured.'
        html_content = html if html else '<p>A Project Activity Occured.</p>'
        msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
        msg.attach_alternative(html_content, "text/html")
        msg.send()

    def is_project_center_administrator(self):
        if self.groups.filter(
            name=settings.PROJECT_CENTER_ADMINISTRATOR_GROUP_NAME).exists():
            return True
        else:
            return False
    def is_project_center_company_administrator(self):
        if self.groups.filter(
            name=settings.PROJECT_CENTER_COMPANY_ADMINISTRATOR_GROUP_NAME).exists():
            return True
        else:
            return False

    def is_project_center_user(self):
        if self.groups.filter(
            name=settings.PROJECT_CENTER_USER_GROUP_NAME).exists():
            return True
        else:
            return False


class ProjectCategory(models.Model):
    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    name = models.CharField(_('Category Name'), max_length=255, blank=False, help_text='Category Name')
    display = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project Category')
        verbose_name_plural = _('Project Categories')

    def __str__(self):
        return self.name


class ProjectStatus(models.Model):
    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    name = models.CharField(_('Status Name'), max_length=255, blank=False, help_text='Status Name')
    display = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project Status')
        verbose_name_plural = _('Project Statuses')

    def __str__(self):
        return self.name


class ProjectStage(models.Model):
    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    name = models.CharField(_('Stage Name'), max_length=255, blank=False, help_text='Stage Name')
    display = models.BooleanField(default=True)

    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project Stage')
        verbose_name_plural = _('Project Stages')

    def __str__(self):
        return self.name


class Project(models.Model):
    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    slug = models.SlugField('Slug', max_length=255, blank=True, null=True, unique=True)
    title = models.CharField(_('Title'), max_length=255, blank=False, help_text='Project Title')
    date = models.DateTimeField(_('Start Date'), blank=True, null=True, help_text='Start Date')
    code = models.CharField(_('Code/ID'), max_length=30, null=True, default=None, blank=True, help_text='Project Code')
    category = models.ForeignKey(ProjectCategory, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    status = models.ForeignKey(ProjectStatus, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    stage = models.ForeignKey(ProjectStage, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    internal = models.BooleanField('Internal', blank=False, default=False, )
    description = models.TextField(_('Description'), blank=True, null=True, default=None,
                                   help_text='Project Description')
    company = models.ForeignKey(Company, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    users = models.ManyToManyField(User, blank=True, null=True, default=None, verbose_name='Project Users', help_text='Users from the Company that are '
                                                                                 'allowed to access the Project')
    last_activity_date = models.DateTimeField(_('Last Activity Date'), blank=True, null=True, help_text='Last Activity Date')
    last_activity_name = models.CharField(_('Last Activity Name'), max_length=255, null=True, default=None, blank=True, help_text='Last Activity Name')



    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project')
        verbose_name_plural = _('Projects')

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.title)
        last = self.last_activity()
        if last:
            self.last_activity_date = last.date if last else None
            self.last_activity_name = last.name if last else None
        super().save(*args, **kwargs)

    def last_activity(self):
        try:
            return self.projectactivity_set.order_by('-date').first()
        except:
            return None

    def category_name(self):
        return self.category.name if self.category else None

    def get_absolute_url(self):
        return reverse('project-detail', args=[self.id])

    def send_group_alert(self, message=None):
        for user in self.users.all():
            print(user)


def activity_file_directory_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/user_<id>/<filename>
    try:
        return 'project_activity_files/{0}/{1}'.format(instance.project.slug, os.path.basename(filename))
    except:
        return None


class ProjectActivity(models.Model):
    import_id = models.IntegerField(_('Import ID'), blank=True, null=True, default=None, help_text='Import ID')
    name = models.CharField(_('Activity Name'), max_length=255, blank=False, help_text='Activity Name')
    user = models.ForeignKey(User, blank=True, null=True, default=None, on_delete=models.SET_NULL)
    project = models.ForeignKey(Project, blank=True, null=True, default=None, on_delete=models.CASCADE)
    date = models.DateTimeField(_('Activity Date'), blank=True, null=True, help_text='Activity Date')
    file = models.FileField(upload_to=activity_file_directory_path, blank=True, null=True, max_length=255)
    notes = models.TextField('Activity Notes', blank=True, null=True)
    reply = models.TextField('Comments/Reply', blank=True, null=True, help_text='*** Adding or changing Comments will '
                                                                                'add a new Activity Record to the '
                                                                                'Project History')
    reply_file = models.FileField('File Attachment', upload_to=activity_file_directory_path, blank=True, null=True,
                                  help_text='*** Attaching a new File will add a new Activity Record to the Project '
                                            'History')



    class Meta(object):
        app_label = 'project_center'
        verbose_name = _('Project Activity')
        verbose_name_plural = _('Project Activities')

    def __str__(self):
        return self.name

    def get_filename(self):
        return os.path.basename(self.file.name) if self.file else None

    def get_mimetype(self):
        return mimetypes.guess_type(self.get_filename())[0] if self.file else None

    def get_suffix(self):
        return pathlib.Path(self.get_filename()).suffix if self.file else None

    def get_download_link(self):
        if self.get_filename():
            return mark_safe(
                '<a href="' + reverse('activity-download', args=[self.id]) + '">' + self.get_filename() + '</a>')
        else:
            return None

    get_download_link.short_description = 'Download Link'

    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "name__icontains",)

    def save(self, *args, **kwargs):
        if self.project and (self.project.last_activity_date != self.date) or (self.project.last_activity_name != self.name):
            project = self.project
            project.last_activity_date = self.date
            project.last_activity_name = self.name
            project.save()
        super().save(*args, **kwargs)



# Generated by Django 4.1.5 on 2023-02-15 11:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0002_user_email_notify'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectactivity',
            name='notes',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-28 19:12

from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import phonenumber_field.modelfields
import project_center.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('title', models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='Title')),
                ('address_1', models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='Address 1')),
                ('city', models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='City')),
                ('postal_code', models.CharField(blank=True, default='', max_length=10, null=True, verbose_name='postal_code')),
                ('zip', models.CharField(blank=True, default='', max_length=10, null=True, verbose_name='postal_code')),
                ('primary_phone', phonenumber_field.modelfields.PhoneNumberField(blank=True, default=None, max_length=50, null=True, region=None, verbose_name='primary_phone')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('hash', models.CharField(max_length=40, verbose_name='hash')),
                ('display', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'Cities',
            },
        ),
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Company Name')),
            ],
            options={
                'verbose_name': 'Company',
                'verbose_name_plural': 'Companies',
            },
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True, verbose_name='Slug')),
                ('title', models.CharField(help_text='Project Title', max_length=255, verbose_name='Title')),
                ('code', models.CharField(blank=True, default=None, help_text='Project Code', max_length=30, null=True, verbose_name='Code/ID')),
                ('internal', models.BooleanField(default=False, verbose_name='Internal')),
                ('description', models.TextField(blank=True, default=None, help_text='Project Description', null=True, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'Project',
                'verbose_name_plural': 'Projects',
            },
        ),
        migrations.CreateModel(
            name='ProjectCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('name', models.CharField(help_text='Category Name', max_length=255, verbose_name='Category Name')),
            ],
            options={
                'verbose_name': 'Project Category',
                'verbose_name_plural': 'Project Categories',
            },
        ),
        migrations.CreateModel(
            name='ProjectStage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('name', models.CharField(help_text='Stage Name', max_length=255, verbose_name='Stage Name')),
            ],
            options={
                'verbose_name': 'Project Stage',
                'verbose_name_plural': 'Project Stages',
            },
        ),
        migrations.CreateModel(
            name='ProjectStatus',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('name', models.CharField(help_text='Status Name', max_length=255, verbose_name='Status Name')),
            ],
            options={
                'verbose_name': 'Project Status',
                'verbose_name_plural': 'Project Statuses',
            },
        ),
        migrations.CreateModel(
            name='State',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('abbrev', models.CharField(blank=True, max_length=2, verbose_name='abbrev')),
                ('hash', models.CharField(blank=True, max_length=40, null=True, verbose_name='hash')),
                ('display', models.BooleanField(default=True)),
                ('atsReference', models.IntegerField(blank=True, default=None, help_text='ATS Reference', null=True, verbose_name='atsReference')),
            ],
            options={
                'verbose_name_plural': 'States',
            },
        ),
        migrations.CreateModel(
            name='Zip',
            fields=[
                ('zipcode', models.CharField(max_length=5, primary_key=True, serialize=False, unique=True)),
                ('approximate_latitude', models.FloatField(blank=True, default=None, null=True)),
                ('approximate_longitude', models.FloatField(blank=True, default=None, null=True)),
                ('population_count', models.IntegerField(blank=True, null=True)),
                ('display', models.BooleanField(default=True)),
                ('city', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='zip', to='project_center.city')),
            ],
        ),
        migrations.CreateModel(
            name='ProjectActivity',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('import_id', models.IntegerField(blank=True, default=None, help_text='Import ID', null=True, verbose_name='Import ID')),
                ('name', models.CharField(help_text='Activity Name', max_length=255, verbose_name='Activity Name')),
                ('date', models.DateTimeField(blank=True, help_text='Activity Date', null=True, verbose_name='Activity Date')),
                ('file', models.FileField(blank=True, null=True, upload_to=project_center.models.activity_file_directory_path)),
                ('project', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.project')),
                ('user', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Project Activity',
                'verbose_name_plural': 'Project Activities',
            },
        ),
        migrations.AddField(
            model_name='project',
            name='category',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.projectcategory'),
        ),
        migrations.AddField(
            model_name='project',
            name='company',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.company'),
        ),
        migrations.AddField(
            model_name='project',
            name='stage',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.projectstage'),
        ),
        migrations.AddField(
            model_name='project',
            name='status',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.projectstatus'),
        ),
        migrations.AddField(
            model_name='project',
            name='users',
            field=models.ManyToManyField(help_text='Users from the Company that are allowed to access the Project', to=settings.AUTH_USER_MODEL, verbose_name='Project Users'),
        ),
        migrations.AddField(
            model_name='city',
            name='state',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.state'),
        ),
        migrations.AddField(
            model_name='user',
            name='company',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.company'),
        ),
        migrations.AddField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='user',
            name='state',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='project_center.state'),
        ),
        migrations.AddField(
            model_name='user',
            name='user_permissions',
            field=models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions'),
        ),
    ]



from django.db.models.signals import pre_save, post_save
from django.core.signals import request_finished
from django.dispatch import receiver
from ..models import ProjectActivity

@receiver(post_save, sender=ProjectActivity)
def duplicate(sender, instance, **kwargs):
    print('foo*******************')
post_save.connect(duplicate, sender=ProjectActivity)



# Generated by Django 4.1.7 on 2023-06-04 12:06

from django.db import migrations, models
import project_center.models


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0009_projectactivity_reply_file_and_more'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Activity',
        ),
        migrations.AddField(
            model_name='project',
            name='last_activity_date',
            field=models.DateTimeField(blank=True, help_text='Last Activity Date', null=True, verbose_name='Last Activity Date'),
        ),
        migrations.AlterField(
            model_name='projectactivity',
            name='reply',
            field=models.TextField(blank=True, help_text='*** Adding or changing Comments will add a new Activity Record to the Project History', null=True, verbose_name='Comments/Reply'),
        ),
        migrations.AlterField(
            model_name='projectactivity',
            name='reply_file',
            field=models.FileField(blank=True, help_text='*** Attaching a new File will add a new Activity Record to the Project History', null=True, upload_to=project_center.models.activity_file_directory_path, verbose_name='File Attachment'),
        ),
    ]



# Generated by Django 4.1.7 on 2023-06-05 17:25

from django.db import migrations, models
import project_center.models


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0011_project_last_activity_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectactivity',
            name='file',
            field=models.FileField(blank=True, max_length=255, null=True, upload_to=project_center.models.activity_file_directory_path),
        ),
    ]



from django.http import HttpResponse, FileResponse, HttpResponseForbidden
from django.views.generic import UpdateView, View, FormView, TemplateView, ListView
from .models import ProjectActivity

class AttachmentDownloadView(View):
    def get(self, request, *args, **kwargs):
        try:
            obj = ProjectActivity.objects.get(id=kwargs['activity_id'])
            if request.user.is_superuser or request.user.is_project_center_administrator():
                return FileResponse(obj.file)
            elif request.user.is_project_center_company_administrator():
                return FileResponse(obj.file)
            elif request.user.is_project_center_user():
                return FileResponse(obj.file)
            else:
                return HttpResponseForbidden()

            # if self.request.user.is_superuser or self.request.user.id == obj.user.id:
            #     return FileResponse(obj.file)
            # else:
            #     return HttpResponseForbidden()
        except:
            return HttpResponseForbidden()